﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

namespace Bonfire
{
    public enum State
    {
        Died,
        Walking,
        AttackR1,
        AttackR1_2,
        Rolling,
        Jump,
        EquipWeapon,
        UnEquipWeapon
    };

    public class PlayerCtrl : MonoBehaviour
    {
        [SerializeField] Text debugText = null;

        // 入力関連
        [SerializeField] float joystickDead = 0.3f;
        float inputHorizontalStickL = 0f;
        float inputVerticalStickL = 0f;

        // 十字キーの入力用フラグ
        bool canInputDpadRight = true;

        // 状態管理
        State state = State.Walking;        // 現在の状態
        State nextState = State.Walking;    // 次の状態
        bool isStateEnd = true;

        // 走っている、全力疾走しているフラグ
        public bool isRunning = false;
        public bool isSprint = false;
        public bool canMove = true;
        public bool canRotate = true;
        public float fixedMoveSpeed = 0;

        // 武器を装備しているかどうかのフラグ
        bool isEquipWeapon = true;

        // xボタンの入力が、ローリングからダッシュになるまでの時間を計算するための変数
        float timeSinceInputFire1 = 0f;

        PlayerStatus status = null;
        PlayerMove playerMove = null;
        HudController hudController = null;


        void Start()
        {
            status = GetComponent<PlayerStatus>();
            playerMove = GetComponent<PlayerMove>();
            hudController = FindObjectOfType<HudController>();
        }

        void Update()
        {
            UpdateTimer();
            inputHorizontalStickL = Input.GetAxis("Horizontal Stick-L");
            inputVerticalStickL = Input.GetAxis("Vertical Stick-L") * -1f; // 上下が逆なので -1 をかけておく

            // 呼び出しが多いので、短い文字数で呼べるようにしておく
            isStateEnd = status.isAnimationEnd;

            // ボタンの処理は、ここに足していく
            float dPad = Input.GetAxis("Horizontal D-Pad");
            if (canInputDpadRight && Mathf.Approximately(dPad, 1f))
            {
                if (isEquipWeapon)
                {
                    nextState = State.UnEquipWeapon;
                }
                else
                {
                    nextState = State.EquipWeapon;
                }
                isEquipWeapon = isEquipWeapon ? false : true;
                canInputDpadRight = false;
            }
            if (Mathf.Approximately(dPad, 0f))
            {
                canInputDpadRight = true;
            }

            if (Input.GetButtonDown("Fire_Share"))
            {
                hudController.toggleHudLeft();
            }

            // 加速処理
            if (Input.GetButton("Fire_1"))
            {
                isRunning = true;
            }
            if (Input.GetButtonUp("Fire_1"))
            {
                isRunning = false;
            }

            // 四角ボタンでローリング
            if (Input.GetButtonDown("Fire_0"))
            {
                if (state != State.Rolling)
                {
                    nextState = State.Rolling;
                }
            }

            // 丸ボタンでジャンプ
            if (Input.GetButtonDown("Fire_2"))
            {
                if (state != State.Jump)
                {
                    nextState = State.Jump;
                }
            }

            if (Input.GetButtonUp("Fire_R1"))
            {
                if (state == State.AttackR1)
                {
                    nextState = State.AttackR1_2;
                }
                else if (state != State.AttackR1 && state != State.AttackR1_2)
                {
                    // AttackR1, AttackR1_2 のときは、次の入力をまだ受け付けない
                    nextState = State.AttackR1;
                }
            }

            // 全力疾走処理
            if (Input.GetButtonDown("Fire_L3"))
            {
                isSprint = isSprint ? false : true;
            }

            // 次の状態に遷移できるようになった時の処理
            if (isStateEnd)
            {
                State prevState = state;
                // StartXX 内で nextState が書き換えられるので、ここで state を更新しておく必要がある
                state = nextState;
                switch (nextState)
                {
                    case State.Walking:
                        StartWalking(prevState);
                        break;
                    case State.AttackR1:
                        StartAttackR1(prevState);
                        break;
                    case State.AttackR1_2:
                        StartAttackR1_2(prevState);
                        break;
                    case State.Rolling:
                        StartRolling(prevState);
                        break;
                    case State.Jump:
                        StartJump(prevState);
                        break;
                    case State.EquipWeapon:
                        StartEquipWeapon(prevState);
                        break;
                    case State.UnEquipWeapon:
                        StartUnEquipWeapon(prevState);
                        break;

                }
            }

            if (debugText != null)
            {
                debugText.text =
                   string.Format("state: {0}\nnexState: {1}\nisStateEnd: {2}\nisGrounded: {3}",
                                state,
                                nextState,
                                isStateEnd,
                                status.isGrounded
                               );
            }

        }

        //
        // コントローラーの入力を返すためのメソッド
        //
        public Vector3 GetPlayerDirection()
        {
            return Vector3.Normalize(new Vector3(inputHorizontalStickL, 0, inputVerticalStickL));
        }

        public float GetMagunitudeStickL()
        {
            float ret = new Vector3(inputHorizontalStickL, 0, inputVerticalStickL).magnitude;
            return ret;
        }

        public bool IsInputStickL()
        {
            float h = inputHorizontalStickL;
            float v = inputVerticalStickL;
            bool ret = ((h > joystickDead) || (h < -joystickDead) || (v > joystickDead) || (v < -joystickDead));
            return ret;
        }

        //
        // 状態(State)を開始するときに行う処理 
        // 
        private void StartWalking(State prevState)
        {
            StateStartCommon();

            // Walking のときだけは isAnimationEnd = true で扱う
            status.isAnimationEnd = true;

            canMove = true;
            canRotate = true;
        }

        private void StartAttackR1(State prevState)
        {
            StateStartCommon();
            status.isAttackR1 = true;

            fixedMoveSpeed = 2f;
        }

        private void StartAttackR1_2(State prevState)
        {
            StateStartCommon();
            status.isAttackR1_2 = true;
        }

        private void StartRolling(State prevState)
        {
            StateStartCommon();
            status.isRolling = true;

            fixedMoveSpeed = 8f;
        }

        private void StartJump(State prevState)
        {
            StateStartCommon();
            status.isJumping = true;

            // ダッシュジャンプの速度
            fixedMoveSpeed = GetMagunitudeStickL() * playerMove.walkSpeed * playerMove.runSpeedUpRatio;

            playerMove.AddJumpPower();
        }

        private void StartEquipWeapon(State prevState)
        {
            StateStartCommon();
            status.EquipWepon = true;
        }

        private void StartUnEquipWeapon(State prevState)
        {
            StateStartCommon();
            status.UnequipWeapon = true;
        }

        //
        // 共通処理のメソッド
        //
        private void StateStartCommon()
        {
            //Debug.Log("StateStartCommon()");
            status.resetStatus();
            status.isAnimationEnd = false;

            nextState = State.Walking;

            canMove = false;
            canRotate = false;

            fixedMoveSpeed = 0;
        }

        //
        // その他のメソッド
        //
        private void UpdateTimer()
        {
            timeSinceInputFire1 += Time.deltaTime;
        }
    }

}
