﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace Bonfire
{
    public class Weapon : MonoBehaviour
    {
        // 武器の装備関連
        [SerializeField] AnimatorOverrideController animatorOverride = null;
        [SerializeField] GameObject weaponBackPrefab = null;
        [SerializeField] GameObject weaponHandMaulPrefab = null;
        [SerializeField] GameObject weaponHandPunchPrefab = null;
        Transform weaponBackPosition = null;
        Transform weaponHandPosition = null;

        Animator animator = null;

        void Start()
        {
            animator = GetComponent<Animator>();

            weaponBackPosition = findTransformInChildren(transform, "BackWeaponPosition");
            weaponHandPosition = findTransformInChildren(transform, "R_HandWeapon");
        }

        // name で指定した オブジェクトを探すメソッド
        Transform findTransformInChildren(Transform tf, string name)
        {
            Transform ret = null;

            if (tf.name == name)
            {
                return tf;
            }
            for (int i = 0; i < tf.childCount; i++)
            {
                ret = findTransformInChildren(tf.GetChild(i), name);
                if (ret != null)
                {
                    return ret;
                }
            }

            return null;
        }

        private void DestroyOldWeapon()
        {
            if (weaponHandPosition.childCount > 0)
            {
                Transform oldWeapon = weaponHandPosition.GetChild(0);
                if (oldWeapon != null)
                {
                    oldWeapon.parent = null;
                    Destroy(oldWeapon.gameObject);
                }
            }

            if (weaponBackPosition.childCount > 0)
            {
                Transform oldBackWeapon = weaponBackPosition.GetChild(0);
                if (oldBackWeapon != null)
                {
                    Destroy(oldBackWeapon.gameObject);
                }
            }
        }

        public void WeponEquipment(bool isArmed)
        {
            DestroyOldWeapon();

            // 武器を装備する
            if (isArmed && weaponHandMaulPrefab != null)
            {
                Instantiate(weaponHandMaulPrefab, weaponHandPosition);
            }

            // 武器を背中に付ける
            if (!isArmed && weaponBackPrefab != null)
            {
                Instantiate(weaponBackPrefab, weaponBackPosition);
            }
            // 拳に当たり判定を付ける
            if (!isArmed && weaponHandPunchPrefab != null)
            {
                Instantiate(weaponHandPunchPrefab, weaponHandPosition);
            }

            // 攻撃側の当たり判定の再設定
            // ※ここでやろうとしたが、Destroy や Instantiage の実際の処理は後で行われる
            // ようなので、AttackArea を生成した時に呼ぶように変更した。
            // attackAreaActivator.SetAttackArea();

            // アニメーションの差し替え
            var overrideController = animator.runtimeAnimatorController as AnimatorOverrideController;
            if (!isArmed && animatorOverride != null)
            {
                animator.runtimeAnimatorController = animatorOverride;
            }
            else if (overrideController != null)
            {
                animator.runtimeAnimatorController = overrideController.runtimeAnimatorController;
            }
        }

    }

}